package com.ho.test.realexamples.test1;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;

import junit.framework.TestCase;

import com.ho.test.realexamples.test1.pojo.Data;
import com.ho.test.realexamples.test1.pojo.FdR;
import com.ho.test.realexamples.test1.pojo.FileRsId;
import com.ho.test.realexamples.test1.pojo.MessageType;
import com.ho.test.realexamples.test1.pojo.PrintInstruction;
import com.ho.test.realexamples.test1.pojo.PrintingTicket;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.io.xml.XmlFriendlyNameCoder;

public class XstreamTest extends TestCase {

    static public final String XML_HEAD = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
    
	
	public void test1() {
		PrintingTicket printingTicket = new PrintingTicket();
		Data data = new Data();
		//
		FdR fdr = new FdR();
		fdr.setFdrNumber("123456");
		fdr.setFdrVersion(1);
		fdr.setFdrStatus("Active");
		fdr.setTitle("TITLE");
		fdr.setDistributionPlace("PARIS");
		fdr.setPrintingPlace("NY");
		fdr.setDistributionPlaceComment("Hello");
		fdr.setDuplication(true);
		fdr.setNotification(true);
		fdr.setDocType("DV");
		fdr.setResponsableName("Huseyin OZVEREN");
		fdr.setResponsableTel("123456");
		fdr.setRequesterCode("MYRQ");
		fdr.setClientDeadline(GregorianCalendar.getInstance());
		//
		List<PrintInstruction> printInstructions = new ArrayList<PrintInstruction>();
		{
			PrintInstruction printInstruction = new PrintInstruction();
			printInstruction.setLanguage("LV");
			printInstruction.setNbCopies(1);
			printInstruction.setPshDeadline(GregorianCalendar.getInstance());
			//
			FileRsId fileRsId = new FileRsId();
			fileRsId.setMapping("HUOMapping");
			fileRsId.setFullRsId("A_FILE1.pdf");
			printInstruction.setFileRsId(fileRsId);
			//
			printInstructions.add(printInstruction);
		}
		{
			PrintInstruction printInstruction = new PrintInstruction();
			printInstruction.setLanguage("RO");
			printInstruction.setNbCopies(1);
			printInstruction.setPshDeadline(GregorianCalendar.getInstance());
			//
			FileRsId fileRsId = new FileRsId();
			fileRsId.setMapping("HUOMappinge");
			fileRsId.setFullRsId("A_FILE2.pdf");
			printInstruction.setFileRsId(fileRsId);
			//
			printInstructions.add(printInstruction);
		}
		fdr.setPrintInstructions(printInstructions);
		data.setFdr(fdr);
		//
		printingTicket.setMessageType(MessageType.TYPE1);
		printingTicket.setApplication("MYAPPLI");
		printingTicket.setData(data);

		//To use XStream, simply instantiate the XStream class:
		//XStream xstream = new XStream();
		//You require xstream-[version].jar, xpp3-[version].jar and xmlpull-[version].jar in the classpath. 
		// 		Xpp3 is a very fast XML pull-parser implementation. If you do not want to include these dependencies, 
		// 		you can use a standard JAXP DOM parser or since Java 6 the integrated StAX parser instead:
		//XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library 
		//XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6

		// creates the xstream
        //XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library starting with Java 6
        // replace $ with "ddd"  
        // replace underscore with underscore  
		// The class XmlFriendlyReplacer is depreceated
		//XmlFriendlyReplacer replacer = new XmlFriendlyReplacer("ddd", "_");  
		XmlFriendlyNameCoder replacer = new XmlFriendlyNameCoder("ddd", "_");
		XStream xstream = new XStream(new DomDriver("UTF-8", replacer)); 


		//xstream.autodetectAnnotations(true);
		xstream.processAnnotations(PrintingTicket.class);
		xstream.processAnnotations(Data.class);
		xstream.processAnnotations(FdR.class);
		xstream.processAnnotations(PrintInstruction.class);
		xstream.processAnnotations(FileRsId.class);

        //Duplicate references are treated as two separate objects and circular references cause an exception
        xstream.setMode(XStream.NO_REFERENCES);


        String xml = null;
        
        // Serialize
		{
			System.out.println("---------------------------------------");

            //xml = xstream.toXML(printingTicket);

			StringWriter writer = new StringWriter();
	        writer.write(XML_HEAD);
			xstream.toXML(printingTicket, writer);
            xml = writer.toString();
            
            // prints the result
            System.out.println(xml);
            
            
		}
		// Deserialize
		{
			System.out.println("---------------------------------------");
            // And now let's try to unmarshal the result shown above:
            // loads the calendar from the string
			PrintingTicket printingTicket2 = (PrintingTicket) xstream.fromXML(xml);
            System.out.println(printingTicket2);
 
            
            //String xml2 = xstream.toXML(printingTicket2);
			StringWriter writer = new StringWriter();
	        writer.write(XML_HEAD);
			xstream.toXML(printingTicket2, writer);
			String xml2 = writer.toString();
            
            // prints the result
            System.out.println(xml2);

            System.out.println("---------------------------------------");
            System.out.println("xml1=xml2 -> "+(xml.equals(xml2)));
		}
		
	}

	
	/**
	 * Method setUp
	 * @throws Exception
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

    

}
