package com.ho.test.realexamples.test2;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

/**
 * custom converter locally for this Calendar, but only for this Calendar, this exact field in this exact type. 
 * @author huseyin
 *
 */
public class SingleValueCalendarConverter implements Converter {

	
	/**
     * Contains the format for storing the date. 
     */
    public final static String DATE_FORMAT = "yyyy-MM-dd\'T\'HH:mm:ss";
    
    
    // -------------------------------------------------------------- PUBLIC METHODS 
    public void marshal(Object source, HierarchicalStreamWriter writer, MarshallingContext context) {
    	// 2012-04-21T04:30:10
    	try{
        	Calendar calendar = (Calendar) source;
        	String str = dateToStr(calendar, null, DATE_FORMAT);
            writer.setValue(str);
    	}catch (Throwable e) {
    		e.printStackTrace();
		}
    }

    public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
    	// 2012-04-21T04:30:10
    	try{
        	Calendar calendar = toCalendar(reader.getValue(), DATE_FORMAT);
        	return calendar;
    	}catch (Throwable e) {
    		e.printStackTrace();
		}
        return null;
    }

    public boolean canConvert(Class type) {
        return type.equals(GregorianCalendar.class);
    }
    
    
    // ------------------------------------------------------------- PRIVATE METHODS 
    private static Calendar toCalendar(String value, String dateFormat) throws ParseException {
        Calendar cal = null;
        DateFormat formatter = new SimpleDateFormat(dateFormat);
        if (value.length() != 0) {
            cal = Calendar.getInstance();
            cal.setTime((Date) formatter.parse(value));
        }
        return cal;
    }
    
    /**
     * Convert a given calendar to a formatted date String (dd/MM/yyyy HH:mm)
     * @param pCalendarToConvert
     * @param pLocale If this parameter is null, the server local is used.
     * @param pDateFormat if this paramater is null, the pLocal is used. 
     *  
     * @return The convert date.
     * @throws ParseException <b><u><i>THIS METHOD MUST THROW A ParseException</i></u></b>
     */
    private static String dateToStr(Calendar pCalendarToConvert, Locale pLocale, String pDateFormat) throws ParseException {
    	if (pCalendarToConvert == null){
    		return null;
    	}
    	
    	Date pDateToConvert = pCalendarToConvert.getTime();
    	if (pDateToConvert==null) {
    		return null;
    	}
        
        if (pDateFormat != null){
            SimpleDateFormat sd = new SimpleDateFormat(pDateFormat);
            return sd.format(pDateToConvert);
        }
    	
    	if (pLocale!=null) {
    		return DateFormat.getDateTimeInstance(DateFormat.SHORT , DateFormat.SHORT, pLocale).format(pDateToConvert);
    	} else {
    		return DateFormat.getDateTimeInstance(DateFormat.SHORT , DateFormat.SHORT).format(pDateToConvert);
    	}
    }
    

}
