package com.ho.test.realexamples.test2;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.util.GregorianCalendar;

import junit.framework.TestCase;

import com.ho.test.realexamples.test1.pojo.FdR;
import com.ho.test.realexamples.test1.pojo.FileRsId;
import com.ho.test.realexamples.test1.pojo.PrintInstruction;
import com.ho.test.realexamples.test2.pojo.Data;
import com.ho.test.realexamples.test2.pojo.JI;
import com.ho.test.realexamples.test2.pojo.MessageType;
import com.ho.test.realexamples.test2.pojo.PrintingTicket;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.io.xml.XmlFriendlyNameCoder;

public class XstreamTest extends TestCase {

    static public final String XML_HEAD = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
    
	
	public void test1() {
		PrintingTicket printingTicket = new PrintingTicket();
		Data data = new Data();
		//
		JI ji = new JI();
		ji.setFdrNumber("808010");
		ji.setFdrVersion(2);
		ji.setJtNumber("155");
		ji.setLanguage("ES");
		ji.setDuplicationLocation("STR");
		ji.setDuplicationDate(GregorianCalendar.getInstance());
		data.setJi(ji);
		//
		printingTicket.setMessageType(MessageType.TYPE1);
		printingTicket.setData(data);

		//To use XStream, simply instantiate the XStream class:
		//XStream xstream = new XStream();
		//You require xstream-[version].jar, xpp3-[version].jar and xmlpull-[version].jar in the classpath. 
		// 		Xpp3 is a very fast XML pull-parser implementation. If you do not want to include these dependencies, 
		// 		you can use a standard JAXP DOM parser or since Java 6 the integrated StAX parser instead:
		//XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library 
		//XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6

		// creates the xstream
        //XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library starting with Java 6
        // replace $ with "ddd"  
        // replace underscore with underscore  
		// The class XmlFriendlyReplacer is depreceated
		//XmlFriendlyReplacer replacer = new XmlFriendlyReplacer("ddd", "_");  
		XmlFriendlyNameCoder replacer = new XmlFriendlyNameCoder("ddd", "_");
		XStream xstream = new XStream(new DomDriver("UTF-8", replacer)); 

		//xstream.autodetectAnnotations(true);
		xstream.processAnnotations(PrintingTicket.class);
		xstream.processAnnotations(Data.class);
		xstream.processAnnotations(FdR.class);
		xstream.processAnnotations(PrintInstruction.class);
		xstream.processAnnotations(FileRsId.class);

        //Duplicate references are treated as two separate objects and circular references cause an exception
        xstream.setMode(XStream.NO_REFERENCES);
        
        String xml = null;
        
        // Serialize
		{
			System.out.println("---------------------------------------");

            //xml = xstream.toXML(printingTicket);

			StringWriter writer = new StringWriter();
	        writer.write(XML_HEAD);
			xstream.toXML(printingTicket, writer);
            xml = writer.toString();
            
            // prints the result
            System.out.println(xml);
            
            
		}
		// Deserialize
		{
			System.out.println("---------------------------------------");
            // And now let's try to unmarshal the result shown above:
            // loads the calendar from the string
			PrintingTicket printingTicket2 = (PrintingTicket) xstream.fromXML(xml);
            System.out.println(printingTicket2);
 
            
            //String xml2 = xstream.toXML(printingTicket2);
			StringWriter writer = new StringWriter();
	        writer.write(XML_HEAD);
			xstream.toXML(printingTicket2, writer);
			String xml2 = writer.toString();
            
            // prints the result
            System.out.println(xml2);

            System.out.println("---------------------------------------");
            System.out.println("xml1=xml2 -> "+(xml.equals(xml2)));
		}
		
		// Deserialize n°1
		{
			try {
				System.out.println("---------------------------------------");
				// And now let's try to unmarshal the result shown above:
				StringBuffer myXml = new StringBuffer("");
				String entryFile = "Example1.xml";

				// Filter FileReader through a Buffered read to read a line at a time
				BufferedReader bufRead = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream(entryFile)));
				//new BufferedReader(new FileReader(entryFile));

				String sCurrentLine;    // String that holds current file line
				// Read through file one line at time. Print line # and line
				while ((sCurrentLine = bufRead.readLine()) != null) {
					myXml.append(sCurrentLine);
				}
				bufRead.close();
				
				// loads the calendar from the string
				PrintingTicket printingTicket2 = (PrintingTicket) xstream.fromXML(myXml.toString());
				System.out.println(printingTicket2);
			} catch (Throwable e) {
				e.printStackTrace();
			}
		}
		
		// Deserialize n°2
		{
			try {
				System.out.println("---------------------------------------");
				// And now let's try to unmarshal the result shown above:
				StringBuffer myXml = new StringBuffer("");
				String entryFile = "Example2.xml";

				// Filter FileReader through a Buffered read to read a line at a time
				BufferedReader bufRead = new BufferedReader(new InputStreamReader(this.getClass().getResourceAsStream(entryFile)));
				//new BufferedReader(new FileReader(entryFile));

				String sCurrentLine;    // String that holds current file line
				// Read through file one line at time. Print line # and line
				while ((sCurrentLine = bufRead.readLine()) != null) {
					myXml.append(sCurrentLine);
				}
				bufRead.close();
				
				// loads the calendar from the string
				PrintingTicket printingTicket2 = (PrintingTicket) xstream.fromXML(myXml.toString());
				System.out.println(printingTicket2);
				
			} catch (Throwable e) {
				e.printStackTrace();
			}
		}
		
		
	}

	
	/**
	 * Method setUp
	 * @throws Exception
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

    

}
