package com.ho.test.xstreamsite.test1simple;

import junit.framework.TestCase;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.StaxDriver;

public class SimpleXstreamTest extends TestCase {
	
	/**
	 * This is a very quick introduction to XStream. Skim read it to get an idea of how simple it is to convert objects to XML and back again. I'm sure you'll have questions afterwards.
	 * 
	 * 
	 * http://xstream.codehaus.org/tutorial.html
	 * 
	 */
	public void test1() {
		
		//To use XStream, simply instantiate the XStream class:
		//XStream xstream = new XStream();
		
		//You require xstream-[version].jar, xpp3-[version].jar and xmlpull-[version].jar in the classpath. 
		// 		Xpp3 is a very fast XML pull-parser implementation. If you do not want to include these dependencies, 
		// 		you can use a standard JAXP DOM parser or since Java 6 the integrated StAX parser instead:
		//XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library 
		//XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
			
		XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6

		// Create aliases for your custom class names to XML element names
		// This is the only type of mapping required to use XStream and even this is optional.
		// This is an optional step. Without it XStream would work fine, 
		// but the XML element names would contain the fully qualified name of each class (including package) which would bulk up the XML a bit.
		xstream.alias("person", Person.class);
		xstream.alias("phonenumber", PhoneNumber.class);
		
		Person joe = new Person("Joe", "Walnes");
		joe.setPhone(new PhoneNumber(123, "1234-456"));
		joe.setFax(new PhoneNumber(123, "9999-999"));
		//
		String xml = null;
		Person newJoe = null;
		
		// Serializing an object to XML
		{
			// To convert it to XML, all you have to do is make a simple call to XStream:
			xml = xstream.toXML(joe);
			System.out.println(xml);

//			<?xml version="1.0" ?>
//			<person>
//				<firstname>Joe</firstname>
//				<lastname>Walnes</lastname>
//				<phone>
//					<code>123</code>
//					<number>1234-456</number>
//				</phone>
//				<fax>
//					<code>123</code>
//					<number>9999-999</number>
//				</fax>
//			</person>

			
		}
		
		
		// Deserializing an object back from XML
		{
			// To reconstruct an object, purely from the XML:
			newJoe = (Person)xstream.fromXML(xml);
			
		}
		assertTrue(newJoe.equals(joe));
	}

	
	/**
	 * Method setUp
	 * @throws Exception
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}


}
