package com.ho.test.xstreamsite.test2alias;

import junit.framework.TestCase;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.SingleValueConverter;
import com.thoughtworks.xstream.io.xml.StaxDriver;

public class AliasXstreamTest extends TestCase {
	
	/**
	 * http://xstream.codehaus.org/alias-tutorial.html
	 * 
	 */
	public void test1() {
		// RESULT TO WANT:
		//		<blog author="Guilherme Silveira">
		//		  <entry>
		//		    <title>first</title>
		//		    <description>My first blog entry.</description>
		//		  </entry>
		//		  <entry>
		//		    <title>tutorial</title>
		//		    <description>
		//		        Today we have developed a nice alias tutorial. Tell your friends! NOW!
		//		    </description>
		//		  </entry>
		//		</blog>		
		

		Blog teamBlog = new Blog(new Author("Guilherme Silveira"));
        teamBlog.add(new Entry("first","My first blog entry."));
        teamBlog.add(new Entry("tutorial", "Today we have developed a nice alias tutorial. Tell your friends! NOW!"));

		// A simple test
		{
	        System.out.println("---------------------------------------");
			XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
	        System.out.println(xstream.toXML(teamBlog));
	        
	        // And the resulting XML is not so nice as we would want it to be:
			//	        <?xml version="1.0" ?>
			//	        <com.ho.test.xstream.test2.Blog>
			//	        	<writer>
			//	        		<name>Guilherme Silveira</name>
			//	        	</writer>
			//	        	<entries>
			//	        		<com.ho.test.xstream.test2.Entry>
			//	        			<title>first</title>
			//	        			<description>My first blog entry.</description>
			//	        		</com.ho.test.xstream.test2.Entry>
			//	        		<com.ho.test.xstream.test2.Entry>
			//	        			<title>tutorial</title>
			//	        			<description>Today we have developed a nice alias tutorial. Tell your
			//	        				friends! NOW!</description>
			//	        		</com.ho.test.xstream.test2.Entry>
			//	        	</entries>
			//	        </com.ho.test.xstream.test2.Blog>
		}
		
		{
	        // Class aliasing
	        // The first thing we shall change is how XStream refers to the com.thoughtworks.xstream.Blog class. We shall name it simply blog: let's create an alias called blog to the desired class:
	        System.out.println("---------------------------------------");
			XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
	        xstream.alias("blog", Blog.class);
	        xstream.alias("entry", Entry.class);
	        System.out.println(xstream.toXML(teamBlog));
			//	        <?xml version="1.0" ?>
			//	        <blog>
			//	        	<writer>
			//	        		<name>Guilherme Silveira</name>
			//	        	</writer>
			//	        	<entries>
			//	        		<entry>
			//	        			<title>first</title>
			//	        			<description>My first blog entry.</description>
			//	        		</entry>
			//	        		<entry>
			//	        			<title>tutorial</title>
			//	        			<description>Today we have developed a nice alias tutorial. Tell your
			//	        				friends! NOW!</description>
			//	        		</entry>
			//	        	</entries>
			//	        </blog>
		}
		
		{
	        // Field aliasing
	        // Next we will change the name of the writer tag, but this time we have to use a field alias:
	        System.out.println("---------------------------------------");
			XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
	        xstream.alias("blog", Blog.class);
	        xstream.alias("entry", Entry.class);
	        xstream.aliasField("author", Blog.class, "writer");
	        System.out.println(xstream.toXML(teamBlog));
			//	        <?xml version="1.0" ?>
			//	        <blog>
			//	        	<author>
			//	        		<name>Guilherme Silveira</name>
			//	        	</author>
			//	        	<entries>
			//	        		<entry>
			//	        			<title>first</title>
			//	        			<description>My first blog entry.</description>
			//	        		</entry>
			//	        		<entry>
			//	        			<title>tutorial</title>
			//	        			<description>Today we have developed a nice alias tutorial. Tell your
			//	        				friends! NOW!</description>
			//	        		</entry>
			//	        	</entries>
			//	        </blog>
		}
		
		{
	        // Implicit Collections
	        // Pay attention to the addImplicitCollection method call: it describes which class and which member variable shall assume the behaviour we described.
	        System.out.println("---------------------------------------");
			XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
	        xstream.alias("blog", Blog.class);
	        xstream.alias("entry", Entry.class);
	        xstream.aliasField("author", Blog.class, "writer");
	        xstream.addImplicitCollection(Blog.class, "entries");
	        System.out.println(xstream.toXML(teamBlog));
			//	        <?xml version="1.0" ?>
			//	        <blog>
			//	        	<author>
			//	        		<name>Guilherme Silveira</name>
			//	        	</author>
			//	        	<entry>
			//	        		<title>first</title>
			//	        		<description>My first blog entry.</description>
			//	        	</entry>
			//	        	<entry>
			//	        		<title>tutorial</title>
			//	        		<description>Today we have developed a nice alias tutorial. Tell your
			//	        			friends! NOW!</description>
			//	        	</entry>
			//	        </blog>
		}
		
		{
	        // Attribute aliasing
	        // The next step is to set the writer member variable as an XML attribute.
	        // In order to do this, we shall tell XStream to alias the writer field of the Blog class as an "author" attribute:
			System.out.println("---------------------------------------");
			XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
	        xstream.alias("blog", Blog.class);
	        xstream.alias("entry", Entry.class);
	        
	        xstream.addImplicitCollection(Blog.class, "entries");
	        
	        xstream.useAttributeFor(Blog.class, "writer");
            xstream.aliasField("author", Blog.class, "writer");
	        
            // And now it leaves us with one problem: how does XStream converts an Author in a String so it can be written as a XML tag attribute?
            // Attributes cannot be written for types that are handled by Converter implementations, we have to use a SingleValueConverter named "AuthorConverter" 
            // and implement our own converter for the Author.
            // Register this converter:
            xstream.registerConverter(new AuthorConverter());

            System.out.println(xstream.toXML(teamBlog));
            //            <?xml version="1.0" ?>
			//            <blog author="Guilherme Silveira">
			//            	<entry>
			//            		<title>first</title>
			//            		<description>My first blog entry.</description>
			//            	</entry>
			//            	<entry>
			//            		<title>tutorial</title>
			//            		<description>Today we have developed a nice alias tutorial. Tell your
			//            			friends! NOW!</description>
			//            	</entry>
			//            </blog>
		}
		
	}

	/**
	 * http://xstream.codehaus.org/alias-tutorial.html
	 * 
	 */
	public void test2() {
		Blog teamBlog = new Blog(new Author("Guilherme Silveira"));
        teamBlog.add(new Entry("first","My first blog entry."));
        teamBlog.add(new Entry("tutorial", "Today we have developed a nice alias tutorial. Tell your friends! NOW!"));

		// Package aliasing
		{
	        System.out.println("---------------------------------------");
	        XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
	        xstream.aliasPackage("my.company", "org.thoughtworks");
	        System.out.println(xstream.toXML(teamBlog));
	        
			//	        <?xml version="1.0" ?>
			//	        <com.ho.test.xstream.test2.Blog>
			//	        	<writer>
			//	        		<name>Guilherme Silveira</name>
			//	        	</writer>
			//	        	<entries>
			//	        		<com.ho.test.xstream.test2.Entry>
			//	        			<title>first</title>
			//	        			<description>My first blog entry.</description>
			//	        		</com.ho.test.xstream.test2.Entry>
			//	        		<com.ho.test.xstream.test2.Entry>
			//	        			<title>tutorial</title>
			//	        			<description>Today we have developed a nice alias tutorial. Tell your
			//	        				friends! NOW!</description>
			//	        		</com.ho.test.xstream.test2.Entry>
			//	        	</entries>
			//	        </com.ho.test.xstream.test2.Blog>
	        
		}
		
	}
	
	
	/**
	 * Method setUp
	 * @throws Exception
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}


}
