package com.ho.test.xstreamsite.test4converter.test3;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import com.thoughtworks.xstream.converters.ConversionException;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

public class DateConverter implements Converter {

	private Locale locale;

	public DateConverter(Locale locale) {
		super();
		this.locale = locale;
	}

	public boolean canConvert(Class clazz) {
		return Calendar.class.isAssignableFrom(clazz);
	}

	public void marshal(Object value, HierarchicalStreamWriter writer, MarshallingContext context) {

		Calendar calendar = (Calendar) value;

		// grabs the date
		Date date = calendar.getTime();

		// grabs the formatter
		DateFormat formatter = DateFormat.getDateInstance(DateFormat.FULL, this.locale);

		// formats and sets the value
		writer.setValue(formatter.format(date));
	}

	public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
        // creates the calendar
        GregorianCalendar calendar = new GregorianCalendar();

        // grabs the converter
        DateFormat formatter = DateFormat.getDateInstance(DateFormat.FULL, this.locale);

        // parses the string and sets the time
        try {
                calendar.setTime(formatter.parse(reader.getValue()));
        } catch (ParseException e) {
                throw new ConversionException(e.getMessage(), e);
        }

        // returns the new object
        return calendar;
	}

}