package com.ho.test.xstreamsite.test4converter.test4;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

import junit.framework.TestCase;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.StaxDriver;

public class ComplexConverterXstreamTest extends TestCase {
	
	/**
	 * http://xstream.codehaus.org/converter-tutorial.html
	 * 
	 */
	public void test1() {
		
		//To use XStream, simply instantiate the XStream class:
		//XStream xstream = new XStream();
		
		//You require xstream-[version].jar, xpp3-[version].jar and xmlpull-[version].jar in the classpath. 
		// 		Xpp3 is a very fast XML pull-parser implementation. If you do not want to include these dependencies, 
		// 		you can use a standard JAXP DOM parser or since Java 6 the integrated StAX parser instead:
		//XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library 
		//XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
			
		// Complex Converter
        // creates the xstream
        XStream xstream = new XStream(new StaxDriver()); // does not require XPP3 library starting with Java 6
        xstream.registerConverter(new BirthdayConverter());
        xstream.registerConverter(new PersonConverter());

        String xml = null;
        
        // Serialize
		{
			System.out.println("---------------------------------------");

			Birthday birthday = new Birthday();
			birthday.setGenderMale();
			//
			Person person = new Person();
	        person.setName("Guilherme");
			birthday.setPerson(person);
			// grabs the current date from the virtual machine
	        Calendar calendar = new GregorianCalendar();
			birthday.setDate(calendar);


            xml = xstream.toXML(birthday);

            // prints the result
            System.out.println(xml);
	        // The final result is:
			//            <?xml version="1.0" ?>
			//            <com.ho.test.xstream.test4converter.test4.Birthday gender="m">
			//            	<person>Guilherme</person>
			//            	<birth>
			//            		<time>1343292314477</time>
			//            		<timezone>Europe/Paris</timezone>
			//            	</birth>
			//            </com.ho.test.xstream.test4converter.test4.Birthday>
		}

		// Deserialize
		{
			System.out.println("---------------------------------------");
            // And now let's try to unmarshal the result shown above:
            // loads the calendar from the string
			Birthday birthday = (Birthday) xstream.fromXML(xml);
            
            // prints using the system defined locale
            System.out.println(birthday);
            System.out.println(DateFormat.getDateInstance(DateFormat.FULL).format(birthday.getDate().getTime()));
            System.out.println(birthday.getPerson());
            System.out.println(birthday.getGender());
 
		}
		
		
	}

	
	/**
	 * Method setUp
	 * @throws Exception
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}


}
